@@ -1,7 +1,7 @@
 package App::riap;
 
-our $DATE = '2020-01-28'; # DATE
-our $VERSION = '0.381'; # VERSION
+our $DATE = '2020-02-05'; # DATE
+our $VERSION = '0.382'; # VERSION
 
 use 5.010001;
 use strict;
@@ -465,8 +465,10 @@ sub _help_cmd {
 }
 
 sub _run_cmd {
-    require Perinci::Sub::GetArgs::Argv;
     require Perinci::Result::Format;
+    require Perinci::Sub::GetArgs::Argv;
+    require Perinci::Sub::ValidateArgs;
+
     local $Perinci::Result::Format::Enable_Cleansing = 1;
 
     my ($self, %args) = @_;
@@ -502,7 +504,17 @@ sub _run_cmd {
             last;
         }
 
-        $res = $args{code}->(%{$res->[2]}, -shell => $self);
+        # validate using schemas in Rinci metadata
+        my $args = $res->[2];
+        $res = Perinci::Sub::ValidateArgs::validate_args_using_meta(
+            args => $args,
+            meta => $args{meta},
+        );
+        unless ($res->[0] == 200) {
+            last;
+        }
+
+        $res = $args{code}->(%$args, -shell => $self);
     }
 
     my $fmt = $opts->{fmt} //
@@ -750,7 +762,7 @@ App::riap - Riap command-line client shell
 
 =head1 VERSION
 
-version 0.381
+version 0.382
 
 =head1 SYNOPSIS
 
